'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _logUpdate = require('log-update');

var _logUpdate2 = _interopRequireDefault(_logUpdate);

var _figures = require('figures');

var _figures2 = _interopRequireDefault(_figures);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const sharedState = {};

const B1 = (0, _figures2.default)('●');

class WebpackBarPlugin extends _webpack2.default.ProgressPlugin {
  constructor(options = { name: 'webpack', color: 'green' }) {
    super(options);
    this.options = options;

    this.handler = (percent, msg, ...details) => this.updateProgress(percent, msg, details);
    this.handler = _lodash2.default.throttle(this.handler, 25, { leading: true, trailing: true });

    this.logUpdate = options.logUpdate || _logUpdate2.default;
  }

  apply(compiler) {
    super.apply(compiler);

    compiler.hooks.done.tap('progress', () => _logUpdate2.default.clear());
  }

  updateProgress(percent, msg, details) {
    const progress = Math.floor(percent * 100);

    if (!sharedState[this.options.name]) {
      sharedState[this.options.name] = {
        color: this.options.color
      };
    }

    const thisState = sharedState[this.options.name];
    thisState.progress = progress;
    thisState.msg = msg;
    thisState.details = details || [];
    thisState.isRunning = progress && progress !== 100 && msg && msg.length;

    // Process all states
    let isRunning = false;

    const lines = [];

    _lodash2.default.sortBy(Object.keys(sharedState), s => s.name).reverse().forEach(name => {
      const state = sharedState[name];

      if (state.isRunning) {
        isRunning = true;
      } else {
        return;
      }

      const lColor = _chalk2.default.keyword(state.color);
      const lIcon = lColor(B1);
      const lName = lColor(_lodash2.default.startCase(name));
      const lBar = (0, _utils.renderBar)(state.progress, state.color);
      const lMsg = _lodash2.default.startCase(state.msg);
      const lProgress = `(${state.progress}%)`;
      const lDetail1 = _chalk2.default.grey((0, _utils.str)(state.details[0]));
      const lDetail2 = _chalk2.default.grey((0, _utils.str)(state.details[1]));
      const lModule = state.details[2] ? _chalk2.default.grey(`  ${(0, _utils.formatModule)(state.details[2])}`) : '';

      lines.push(`${[lIcon, lName, lBar, lMsg, lProgress, lDetail1, lDetail2].join(' ')}\n${lModule}`);
    });

    if (!isRunning) {
      this.logUpdate.clear();
    } else {
      const title = _chalk2.default.bgBlue.black('BUILDING');
      this.logUpdate(`\n${title}\n\n${lines.join('\n\n')}`);
    }
  }
}
exports.default = WebpackBarPlugin;